extern crate regex;
use regex::Regex;

use std::str::FromStr;

#[derive(Debug)]
pub struct Header {
    code: i16,
    meta: String,
}

#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
pub struct ParseError;

const HEADER_REGEX: &str = r"^(\d{1,3})[ \t](.*)$";

impl FromStr for Header {
    type Err = ParseError;

    // Parses a &str into an instance of 'Header'
    fn from_str(line: &str) -> Result<Self, Self::Err> {
        let header_regexp = Regex::new(HEADER_REGEX).unwrap();

        match header_regexp.captures(&line) {
            Some(caps) => {
                let code_str = caps.get(1).map_or("", |m| m.as_str());
                let meta_str = caps.get(2).map_or("", |m| m.as_str());

                let code: i16 = code_str.parse().expect("2");
                let meta = meta_str.to_string();

                Ok(Header { code, meta })
            }
            None => Err(ParseError),
        }
    }
}
