extern crate regex;
use regex::Regex;

use std::str::FromStr;

#[derive(Debug)]
pub struct Link {
    pub url: String,
    pub label: String,
}

#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
pub struct ParseError;

const LINK_REGEX: &str = r"^=>\s(\S*)\s*(.*)?$";

impl FromStr for Link {
    type Err = ParseError;

    // Parses a &str into an instance of 'Link'
    fn from_str(line: &str) -> Result<Self, Self::Err> {
        let link_regexp = Regex::new(LINK_REGEX).unwrap();

        match link_regexp.captures(&line) {
            Some(caps) => {
                let url_str = caps.get(1).map_or("", |m| m.as_str());
                let label_str = caps.get(2).map_or("", |m| m.as_str());

                let url = url_str.to_string();
                let label = if label_str.is_empty() {
                    url_str.to_string()
                } else {
                    label_str.to_string()
                };

                Ok(Link { url, label })
            }
            None => Err(ParseError),
        }
    }
}
